| bsmote | R Documentation |
BSMOTE generates new examples of the minority class using nearest neighbors of these cases in the border region between classes.
bsmote(
df,
var,
k = 5,
over_ratio = 1,
all_neighbors = FALSE,
distance = "euclidean"
)
df |
data.frame or tibble. Must have 1 factor variable and remaining numeric variables. |
var |
Character, name of variable containing factor variable. |
k |
An integer. Number of nearest neighbor that are used to generate the new examples of the minority class. |
over_ratio |
A numeric value for the ratio of the minority-to-majority frequencies. The default value (1) means that all other levels are sampled up to have the same frequency as the most occurring level. A value of 0.5 would mean that the minority levels will have (at most) (approximately) half as many rows as the majority level. A named numeric vector can be used instead to give different levels
different targets, for example |
all_neighbors |
Type of two borderline-SMOTE method. Defaults to FALSE. See details. |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
BSMOTE (borderline-SMOTE) works the same way as SMOTE, except that instead of generating points around every point of the minority class each point is first classified into the boxes "danger" and "not". For each point the nearest neighbors are calculated. If all the neighbors come from a different class it is labeled noise and put into the "not" box. If more than half of the neighbors come from a different class it is labeled "danger". Points are generated around points labeled "danger".
If all_neighbors = FALSE then points are generated between nearest
neighbors in its own class. If all_neighbors = TRUE then points are
generated between any nearest neighbors. See examples for visualization.
SMOTE generates new examples of the minority class using nearest neighbors
of these cases. For each existing minority class example, new examples are
created by interpolating between the example and its nearest neighbors. The
number of nearest neighbors used is controlled by the number of neighbors
argument (k in smote(), neighbors in step_smote()), and the number
of new examples generated is controlled by over_ratio.
All columns used in this function must be numeric with no missing data.
A data.frame or tibble, depending on type of df.
Hui Han, Wen-Yuan Wang, and Bing-Huan Mao. Borderline-smote: a new over-sampling method in imbalanced data sets learning. In International Conference on Intelligent Computing, pages 878–887. Springer, 2005.
step_bsmote() for step function of this method
Other Direct Implementations:
adasyn(),
cluster_centroids(),
cnn(),
enn(),
instance_hardness(),
kmeans_smote(),
ncl(),
nearmiss(),
oss(),
rose(),
smogn(),
smote(),
smoten(),
smotenc(),
svmsmote(),
tomek()
circle_numeric <- circle_example[, c("x", "y", "class")]
res <- bsmote(circle_numeric, var = "class")
res <- bsmote(circle_numeric, var = "class", k = 10)
res <- bsmote(circle_numeric, var = "class", over_ratio = 0.8)
res <- bsmote(circle_numeric, var = "class", all_neighbors = TRUE)
res <- bsmote(circle_numeric, var = "class", distance = "manhattan")
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.