S3 methods for tracking which additional packages are needed for steps.
Source:R/adasyn.R, R/bsmote.R, R/cluster_centroids.R, and 16 more
required_pkgs.step.RdS3 methods for tracking which additional packages are needed for steps.
Usage
# S3 method for class 'step_adasyn'
required_pkgs(x, ...)
# S3 method for class 'step_bsmote'
required_pkgs(x, ...)
# S3 method for class 'step_cluster_centroids'
required_pkgs(x, ...)
# S3 method for class 'step_cnn'
required_pkgs(x, ...)
# S3 method for class 'step_downsample'
required_pkgs(x, ...)
# S3 method for class 'step_enn'
required_pkgs(x, ...)
# S3 method for class 'step_instance_hardness'
required_pkgs(x, ...)
# S3 method for class 'step_kmeans_smote'
required_pkgs(x, ...)
# S3 method for class 'step_ncl'
required_pkgs(x, ...)
# S3 method for class 'step_nearmiss'
required_pkgs(x, ...)
# S3 method for class 'step_oss'
required_pkgs(x, ...)
# S3 method for class 'step_rose'
required_pkgs(x, ...)
# S3 method for class 'step_smogn'
required_pkgs(x, ...)
# S3 method for class 'step_smote'
required_pkgs(x, ...)
# S3 method for class 'step_smoten'
required_pkgs(x, ...)
# S3 method for class 'step_smotenc'
required_pkgs(x, ...)
# S3 method for class 'step_svmsmote'
required_pkgs(x, ...)
# S3 method for class 'step_tomek'
required_pkgs(x, ...)
# S3 method for class 'step_upsample'
required_pkgs(x, ...)