| step_cnn | R Documentation |
step_cnn() creates a specification of a recipe step that removes
redundant majority class observations, keeping only a consistent subset that
correctly classifies the data using a 1-nearest-neighbor rule.
step_cnn(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
distance = "euclidean",
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("cnn")
)
recipe |
A recipe object. The step will be added to the sequence of operations for this recipe. |
... |
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the |
role |
Not used by this step since no new variables are created. |
trained |
A logical to indicate if the quantities for preprocessing have been estimated. |
column |
A character string of the variable name that will
be populated (eventually) by the |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
skip |
A logical. Should the step be skipped when the recipe is baked by
|
seed |
An integer that will be used as the seed when applied. |
distance_with |
A call to a selector function to choose
which variables are used for distance calculations. Defaults to
|
id |
A character string that is unique to this step to identify it. |
Condensed Nearest Neighbors (CNN) is an under-sampling method that reduces the majority classes to a consistent subset: a subset that classifies the original data correctly using a 1-nearest-neighbor rule. It starts with a "store" containing all minority class observations and one randomly chosen majority class observation. It then repeatedly scans the remaining majority class observations and moves any that are misclassified by a 1-nearest neighbor fit on the current store into the store. This continues until a full pass adds no new observations. The observations left outside the store are removed.
The smallest class is treated as the minority class and is always kept. CNN tends to keep observations near the decision boundary while discarding redundant interior observations. Because the seed observation and the scan order are random, results depend on the random seed.
All variables selected by distance_with must be numeric with no missing
data.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
When you tidy() this step, a tibble is returned with
columns terms and id:
character, the selectors or variables selected
character, id of this step
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
Hart, P. (1968). The condensed nearest neighbor rule. IEEE Transactions on Information Theory, 14(3), 515-516.
cnn() for direct implementation
Other Steps for under-sampling:
step_cluster_centroids(),
step_downsample(),
step_enn(),
step_instance_hardness(),
step_ncl(),
step_nearmiss(),
step_oss(),
step_tomek()
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
up_rec <- recipe(class ~ ., data = hpc_data0) |>
step_cnn(class) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without CNN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_cnn(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With CNN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.