| step_enn | R Documentation |
step_enn() creates a specification of a recipe step that removes
observations whose class differs from the majority of their nearest
neighbors.
step_enn(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
neighbors = 3,
distance = "euclidean",
times = 1,
all_k = FALSE,
kind_sel = "mode",
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("enn")
)
recipe |
A recipe object. The step will be added to the sequence of operations for this recipe. |
... |
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the |
role |
Not used by this step since no new variables are created. |
trained |
A logical to indicate if the quantities for preprocessing have been estimated. |
column |
A character string of the variable name that will
be populated (eventually) by the |
neighbors |
An integer. Number of nearest neighbor that are used to
decide whether an observation is removed. Defaults to |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
times |
A positive integer for the maximum number of times ENN is
applied. Defaults to |
all_k |
A logical. When |
kind_sel |
A character string. The rule used to decide whether an
observation is removed. |
skip |
A logical. Should the step be skipped when the recipe is baked by
|
seed |
An integer that will be used as the seed when applied. |
distance_with |
A call to a selector function to choose
which variables are used for distance calculations. Defaults to
|
id |
A character string that is unique to this step to identify it. |
Edited Nearest Neighbors (ENN) is a cleaning method. For each observation it
finds the neighbors nearest neighbors and, if the class of the observation
does not match the majority class among those neighbors, the observation is
removed. This tends to remove noisy and borderline observations, which can
lead to smoother decision boundaries.
Setting times greater than 1 applies ENN repeatedly, removing more noisy
and borderline observations on each pass and stopping early once a pass
removes nothing. This corresponds to Repeated Edited Nearest Neighbors
(RENN).
Setting all_k = TRUE applies ENN with increasing numbers of neighbors, from
1 up to neighbors, cleaning the data at each step. This corresponds to
All k-Nearest Neighbors (AllKNN) and takes precedence over times.
Setting kind_sel = "all" uses a stricter cleaning rule: instead of removing
an observation when the majority of its neighbors disagree, it is removed
unless every one of its neighbors shares its class. This removes more
observations than the default kind_sel = "mode".
All variables selected by distance_with must be numeric with no missing
data.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
The data must have at least neighbors + 1 observations for the nearest
neighbors to be computed.
When you tidy() this step, a tibble is returned with
columns terms and id:
character, the selectors or variables selected
character, id of this step
This step has 2 tuning parameters:
neighbors: # Nearest Neighbors (type: integer, default: 3)
all_k: Pruning (type: logical, default: FALSE)
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
Wilson, D. L. (1972). Asymptotic properties of nearest neighbor rules using edited data. IEEE Transactions on Systems, Man, and Cybernetics, (3), 408-421.
Tomek, I. (1976). An experiment with the edited nearest-neighbor rule. IEEE Transactions on Systems, Man, and Cybernetics, (6), 448-452.
enn() for direct implementation
step_smote(), which is commonly composed before step_enn() to clean the
ambiguous points that over-sampling creates near the class boundary (the
equivalent of imbalanced-learn's SMOTEENN).
Other Steps for under-sampling:
step_cluster_centroids(),
step_cnn(),
step_downsample(),
step_instance_hardness(),
step_ncl(),
step_nearmiss(),
step_oss(),
step_tomek()
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
up_rec <- recipe(class ~ ., data = hpc_data0) |>
step_enn(class) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without ENN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_enn(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With ENN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.