View source: R/instance_hardness.R
| step_instance_hardness | R Documentation |
step_instance_hardness() creates a specification of a recipe step that
removes majority class instances by under-sampling the points that are
hardest to classify.
step_instance_hardness(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
under_ratio = 1,
neighbors = 5,
distance = "euclidean",
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("instance_hardness")
)
recipe |
A recipe object. The step will be added to the sequence of operations for this recipe. |
... |
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the |
role |
Not used by this step since no new variables are created. |
trained |
A logical to indicate if the quantities for preprocessing have been estimated. |
column |
A character string of the variable name that will
be populated (eventually) by the |
under_ratio |
A numeric value for the ratio of the majority-to-minority frequencies. The default value (1) means that all other levels are sampled down to have the same frequency as the least occurring level. A value of 2 would mean that the majority levels will have (at most) (approximately) twice as many rows than the minority level. A named numeric vector can be used instead to give different levels
different targets, for example |
neighbors |
An integer. Number of nearest neighbor that are used to generate the new examples of the minority class. |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
skip |
A logical. Should the step be skipped when the recipe is baked by
|
seed |
An integer that will be used as the seed when applied. |
distance_with |
A call to a selector function to choose
which variables are used for distance calculations. Defaults to
|
id |
A character string that is unique to this step to identify it. |
The instance hardness of each observation is estimated using the
k-Disagreeing Neighbors measure: the proportion of the nearest neighbors
that belong to a different class. Observations that are surrounded by points
of a different class are considered hard to classify. For each majority
class, the hardest observations are removed until the desired under_ratio
is reached.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
All columns selected by distance_with must be numeric with no missing
data.
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
The data must have at least neighbors + 1 observations for the nearest
neighbors to be computed.
When you tidy() this step, a tibble is returned with
columns terms and id:
character, the selectors or variables selected
character, id of this step
This step has 2 tuning parameters:
under_ratio: Under-Sampling Ratio (type: double, default: 1)
neighbors: # Nearest Neighbors (type: integer, default: 5)
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
Smith, M. R., Martinez, T., & Giraud-Carrier, C. (2014). An instance level analysis of data complexity. Machine learning, 95(2), 225-256.
instance_hardness() for direct implementation
Other Steps for under-sampling:
step_cluster_centroids(),
step_cnn(),
step_downsample(),
step_enn(),
step_ncl(),
step_nearmiss(),
step_oss(),
step_tomek()
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
up_rec <- recipe(class ~ ., data = hpc_data0) |>
# Bring the majority levels down to about 1000 each
# 1000/259 is approx 3.862
step_instance_hardness(class, under_ratio = 3.862) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
# Note that if the original data contained fewer rows than the
# target n (= ratio * minority_n), the data are left alone:
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without instance hardness") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_instance_hardness(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With instance hardness") +
xlim(c(1, 15)) +
ylim(c(1, 15))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.