| step_ncl | R Documentation |
step_ncl() creates a specification of a recipe step that removes majority
class observations that are noisy or that pollute the neighborhood of
minority class observations.
step_ncl(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
neighbors = 3,
distance = "euclidean",
threshold_clean = 0.5,
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("ncl")
)
recipe |
A recipe object. The step will be added to the sequence of operations for this recipe. |
... |
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the |
role |
Not used by this step since no new variables are created. |
trained |
A logical to indicate if the quantities for preprocessing have been estimated. |
column |
A character string of the variable name that will
be populated (eventually) by the |
neighbors |
An integer. Number of nearest neighbor that are used to
decide whether an observation is removed. Defaults to |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
threshold_clean |
A numeric. Majority classes are only cleaned around
minority class observations when their size is greater than
|
skip |
A logical. Should the step be skipped when the recipe is baked by
|
seed |
An integer that will be used as the seed when applied. |
distance_with |
A call to a selector function to choose
which variables are used for distance calculations. Defaults to
|
id |
A character string that is unique to this step to identify it. |
The Neighborhood Cleaning Rule (NCL) is a cleaning method that combines two
passes over the data. First, it applies the Edited Nearest Neighbors rule,
removing majority class observations whose class differs from the majority of
their neighbors nearest neighbors. Second, for each minority class
observation that is itself misclassified by its neighbors, the majority class
observations among those neighbors are removed. Compared to Edited Nearest
Neighbors, this focuses the cleaning on the neighborhoods of minority class
observations.
The smallest class is treated as the minority class. Only majority classes
larger than threshold_clean times the size of the minority class are
cleaned in the second pass.
All variables selected by distance_with must be numeric with no missing
data.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
The data must have at least neighbors + 1 observations for the nearest
neighbors to be computed.
When you tidy() this step, a tibble is returned with
columns terms and id:
character, the selectors or variables selected
character, id of this step
This step has 2 tuning parameters:
neighbors: # Nearest Neighbors (type: integer, default: 3)
threshold_clean: Threshold (type: double, default: 0.5)
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
Laurikkala, J. (2001). Improving identification of difficult small classes by balancing class distribution. In Conference on Artificial Intelligence in Medicine in Europe (pp. 63-66). Springer.
ncl() for direct implementation
Other Steps for under-sampling:
step_cluster_centroids(),
step_cnn(),
step_downsample(),
step_enn(),
step_instance_hardness(),
step_nearmiss(),
step_oss(),
step_tomek()
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
up_rec <- recipe(class ~ ., data = hpc_data0) |>
step_ncl(class) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without NCL") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_ncl(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With NCL") +
xlim(c(1, 15)) +
ylim(c(1, 15))
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.