| step_smote | R Documentation |
step_smote() creates a specification of a recipe step that generate new
examples of the minority class using nearest neighbors of these cases.
step_smote(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
over_ratio = 1,
neighbors = 5,
distance = "euclidean",
indicator_column = NULL,
skip = TRUE,
seed = sample.int(10^5, 1),
id = rand_id("smote")
)
recipe |
A recipe object. The step will be added to the sequence of operations for this recipe. |
... |
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the |
role |
Not used by this step since no new variables are created. |
trained |
A logical to indicate if the quantities for preprocessing have been estimated. |
column |
A character string of the variable name that will
be populated (eventually) by the |
over_ratio |
A numeric value for the ratio of the minority-to-majority frequencies. The default value (1) means that all other levels are sampled up to have the same frequency as the most occurring level. A value of 0.5 would mean that the minority levels will have (at most) (approximately) half as many rows as the majority level. A named numeric vector can be used instead to give different levels
different targets, for example |
neighbors |
An integer. Number of nearest neighbor that are used to generate the new examples of the minority class. |
distance |
A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to
The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors. |
indicator_column |
A single string or |
skip |
A logical. Should the step be skipped when the recipe is baked by
|
seed |
An integer that will be used as the seed when applied. |
id |
A character string that is unique to this step to identify it. |
SMOTE generates new examples of the minority class using nearest neighbors
of these cases. For each existing minority class example, new examples are
created by interpolating between the example and its nearest neighbors. The
number of nearest neighbors used is controlled by the number of neighbors
argument (k in smote(), neighbors in step_smote()), and the number
of new examples generated is controlled by over_ratio.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
All columns used in this step must be numeric with no missing data.
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
Each minority class must have at least neighbors + 1 observations to
perform the SMOTE algorithm.
When you tidy() this step, a tibble is returned with
columns terms and id:
character, the selectors or variables selected
character, id of this step
This step has 2 tuning parameters:
over_ratio: Over-Sampling Ratio (type: double, default: 1)
neighbors: # Nearest Neighbors (type: integer, default: 5)
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
Chawla, N. V., Bowyer, K. W., Hall, L. O., and Kegelmeyer, W. P. (2002). Smote: Synthetic minority over-sampling technique. Journal of Artificial Intelligence Research, 16:321-357.
smote() for direct implementation
step_enn() and step_tomek(), which are commonly composed after
step_smote() to clean the ambiguous points that over-sampling creates
near the class boundary (the equivalent of imbalanced-learn's SMOTEENN
and SMOTETomek).
Other Steps for over-sampling:
step_adasyn(),
step_bsmote(),
step_kmeans_smote(),
step_rose(),
step_smogn(),
step_smoten(),
step_smotenc(),
step_svmsmote(),
step_upsample()
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
up_rec <- recipe(class ~ ., data = hpc_data0) |>
# Bring the minority levels up to about 1000 each
# 1000/2211 is approx 0.4523
step_smote(class, over_ratio = 0.4523) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
# Note that if the original data contained more rows than the
# target n (= ratio * majority_n), the data are left alone:
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
# A named vector gives each level its own target. Here "VF" is left
# untouched and only "L" is brought up to the size of the majority level.
recipe(class ~ ., data = hpc_data0) |>
step_smote(class, over_ratio = c(L = 1)) |>
prep() |>
bake(new_data = NULL) |>
count(class)
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without SMOTE")
recipe(class ~ x + y, data = circle_example) |>
step_smote(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With SMOTE")
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.