Generates synthetic positive instances using ADASYN algorithm.
Arguments
- df
data.frame or tibble. Must have 1 factor variable and remaining numeric variables.
- var
Character, name of variable containing factor variable.
- k
An integer. Number of nearest neighbor that are used to generate the new examples of the minority class.
- over_ratio
A numeric value for the ratio of the minority-to-majority frequencies. The default value (1) means that all other levels are sampled up to have the same frequency as the most occurring level. A value of 0.5 would mean that the minority levels will have (at most) (approximately) half as many rows as the majority level. See
vignette("ratio", package = "themis")for more details.
References
He, H., Bai, Y., Garcia, E. and Li, S. 2008. ADASYN: Adaptive synthetic sampling approach for imbalanced learning. Proceedings of IJCNN 2008. (IEEE World Congress on Computational Intelligence). IEEE International Joint Conference. pp.1322-1328.
See also
step_adasyn() for step function of this method
Other Direct Implementations:
bsmote(),
nearmiss(),
rose(),
smote(),
smotenc(),
tomek()
Examples
circle_numeric <- circle_example[, c("x", "y", "class")]
res <- adasyn(circle_numeric, var = "class")
res <- adasyn(circle_numeric, var = "class", k = 10)
res <- adasyn(circle_numeric, var = "class", over_ratio = 0.8)