A random dataset with two classes one of which is inside a circle. Used for
examples to show how the different methods handles borders.
A data frame with 400 rows and 4 variables:
- x
Numeric.
- y
Numeric.
- class
Factor, values "Circle" and "Rest".
- id
character, ID variable.
Synthetic data, simulated in data-raw/circle_example.R.