Skip to content

SMOTEN generates new examples of the minority class using nearest neighbors of these cases, for data sets where all predictors are categorical.

Usage

smoten(df, var, k = 5, over_ratio = 1)

Arguments

df

data.frame or tibble. Must have 1 factor variable used as the outcome and remaining categorical (factor or character) variables.

var

Character, name of variable containing factor variable.

k

An integer. Number of nearest neighbor that are used to generate the new examples of the minority class.

over_ratio

A numeric value for the ratio of the minority-to-majority frequencies. The default value (1) means that all other levels are sampled up to have the same frequency as the most occurring level. A value of 0.5 would mean that the minority levels will have (at most) (approximately) half as many rows as the majority level.

A named numeric vector can be used instead to give different levels different targets, for example c(a = 1, b = 0.5). The names must be levels of the outcome and the values are ratios of the majority level, exactly as in the single-number case. Levels that are not named are left untouched, as are rows with a missing outcome. Because a vector of targets is not a single value, supplying one means this argument can no longer be tuned. See vignette("ratio", package = "themis") for more details.

Value

A data.frame or tibble, depending on type of df.

Details

SMOTEN is a variant of SMOTE designed for data sets where all predictors are categorical (nominal). Since standard SMOTE relies on interpolation in continuous space, it cannot be applied to categorical features directly. SMOTEN instead uses the Value Difference Metric (VDM) to measure the distance between observations. For each minority class example, new synthetic examples are generated by taking the most common value of each predictor among its nearest neighbors. The number of new examples generated is controlled by over_ratio.

All columns other than var must be categorical (factor or character) with no missing data.

References

Chawla, N. V., Bowyer, K. W., Hall, L. O., and Kegelmeyer, W. P. (2002). Smote: Synthetic minority over-sampling technique. Journal of Artificial Intelligence Research, 16:321-357.

See also

Examples

df <- data.frame(
  x = factor(sample(letters[1:3], 100, replace = TRUE)),
  y = factor(sample(letters[1:2], 100, replace = TRUE)),
  class = factor(c(rep("minority", 20), rep("majority", 80)))
)

res <- smoten(df, var = "class")

res <- smoten(df, var = "class", k = 10)

res <- smoten(df, var = "class", over_ratio = 0.8)