step_cnn() creates a specification of a recipe step that removes
redundant majority class observations, keeping only a consistent subset that
correctly classifies the data using a 1-nearest-neighbor rule.
Usage
step_cnn(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
distance = "euclidean",
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("cnn")
)Arguments
- recipe
A recipe object. The step will be added to the sequence of operations for this recipe.
- ...
One or more selector functions to choose which variable is used to sample the data. See recipes::selections for more details. The selection should result in single factor variable. For the
tidymethod, these are not currently used.- role
Not used by this step since no new variables are created.
- trained
A logical to indicate if the quantities for preprocessing have been estimated.
- column
A character string of the variable name that will be populated (eventually) by the
...selectors.- distance
A character string specifying the distance metric used for nearest neighbor calculations, defaulting to
"euclidean". The available metrics fall into three groups."euclidean","cosine", and"mahalanobis"use approximate nearest neighbors via the RANN package and scale well to large datasets."squared_chord","matusita","hellinger", and"bhattacharyya"are probability-divergence measures that treat each row as a distribution over the predictors, so they require non-negative values."hellinger"and"bhattacharyya"further require each row to sum to 1. All four also use the RANN package and scale well to large datasets."manhattan","chebyshev","canberra","soergel","lorentzian","jeffreys","topsoe","jensen-shannon","jensen_difference","taneja", and"kumar-johnson"compute an exact all-pairs distance matrix. This takes time and memory proportional to the square of the number of observations in a class, so these are best suited to smaller datasets. Everything from"canberra"onwards is a probability divergence requiring non-negative values, is provided by the philentropy package (which must be installed separately), and in the case of"jeffreys","taneja", and"kumar-johnson"requires strictly positive values, since those divide by individual predictor values.The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors.
- skip
A logical. Should the step be skipped when the recipe is baked by
bake()? While all operations are baked whenprep()is run, some operations may not be able to be conducted on new data (e.g. processing the outcome variable(s)). Care should be taken when usingskip = TRUEas it may affect the computations for subsequent operations.- seed
An integer that will be used as the seed when applied.
- distance_with
A call to a selector function to choose which variables are used for distance calculations. Defaults to
recipes::all_predictors(). The variable selected by...is always excluded from the distance calculations.- id
A character string that is unique to this step to identify it.
Value
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
Details
Condensed Nearest Neighbors (CNN) is an under-sampling method that reduces the majority classes to a consistent subset: a subset that classifies the original data correctly using a 1-nearest-neighbor rule. It starts with a "store" containing all minority class observations and one randomly chosen majority class observation. It then repeatedly scans the remaining majority class observations and moves any that are misclassified by a 1-nearest neighbor fit on the current store into the store. This continues until a full pass adds no new observations. The observations left outside the store are removed.
The smallest class is treated as the minority class and is always kept. CNN tends to keep observations near the decision boundary while discarding redundant interior observations. Because the seed observation and the scan order are random, results depend on the random seed.
All variables selected by distance_with must be numeric with no missing
data.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
Tidying
When you tidy() this step, a tibble is returned with
columns terms and id:
- terms
character, the selectors or variables selected
- id
character, id of this step
Case weights
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
References
Hart, P. (1968). The condensed nearest neighbor rule. IEEE Transactions on Information Theory, 14(3), 515-516.
See also
cnn() for direct implementation
Other Steps for under-sampling:
step_cluster_centroids(),
step_downsample(),
step_enn(),
step_instance_hardness(),
step_ncl(),
step_nearmiss(),
step_oss(),
step_tomek()
Examples
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
#> # A tibble: 4 × 2
#> class orig
#> <fct> <int>
#> 1 VF 2211
#> 2 F 1347
#> 3 M 514
#> 4 L 259
up_rec <- recipe(class ~ ., data = hpc_data0) |>
step_cnn(class) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
#> # A tibble: 4 × 2
#> class training
#> <fct> <int>
#> 1 VF 933
#> 2 F 803
#> 3 M 339
#> 4 L 259
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
#> # A tibble: 4 × 2
#> class baked
#> <fct> <int>
#> 1 VF 2211
#> 2 F 1347
#> 3 M 514
#> 4 L 259
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
#> # A tibble: 4 × 4
#> class orig training baked
#> <fct> <int> <int> <int>
#> 1 VF 2211 933 2211
#> 2 F 1347 803 1347
#> 3 M 514 339 514
#> 4 L 259 259 259
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without CNN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_cnn(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With CNN") +
xlim(c(1, 15)) +
ylim(c(1, 15))