step_enn() creates a specification of a recipe step that removes
observations whose class differs from the majority of their nearest
neighbors.
Usage
step_enn(
recipe,
...,
role = NA,
trained = FALSE,
column = NULL,
neighbors = 3,
distance = "euclidean",
times = 1,
all_k = FALSE,
kind_sel = "mode",
skip = TRUE,
seed = sample.int(10^5, 1),
distance_with = recipes::all_predictors(),
id = rand_id("enn")
)Arguments
- recipe
A recipe object. The step will be added to the sequence of operations for this recipe.
- ...
One or more selector functions to choose which variable is used to sample the data. See recipes::selections for more details. The selection should result in single factor variable. For the
tidymethod, these are not currently used.- role
Not used by this step since no new variables are created.
- trained
A logical to indicate if the quantities for preprocessing have been estimated.
- column
A character string of the variable name that will be populated (eventually) by the
...selectors.- neighbors
An integer. Number of nearest neighbor that are used to decide whether an observation is removed. Defaults to
3, unlike the over-sampling steps which default to5.- distance
A character string specifying the distance metric used for nearest neighbor calculations, defaulting to
"euclidean". The available metrics fall into three groups."euclidean","cosine", and"mahalanobis"use approximate nearest neighbors via the RANN package and scale well to large datasets."squared_chord","matusita","hellinger", and"bhattacharyya"are probability-divergence measures that treat each row as a distribution over the predictors, so they require non-negative values."hellinger"and"bhattacharyya"further require each row to sum to 1. All four also use the RANN package and scale well to large datasets."manhattan","chebyshev","canberra","soergel","lorentzian","jeffreys","topsoe","jensen-shannon","jensen_difference","taneja", and"kumar-johnson"compute an exact all-pairs distance matrix. This takes time and memory proportional to the square of the number of observations in a class, so these are best suited to smaller datasets. Everything from"canberra"onwards is a probability divergence requiring non-negative values, is provided by the philentropy package (which must be installed separately), and in the case of"jeffreys","taneja", and"kumar-johnson"requires strictly positive values, since those divide by individual predictor values.The probability divergences are meaningful for compositional predictors such as proportions or counts normalized per observation, and are generally not appropriate for standardized predictors.
- times
A positive integer for the maximum number of times ENN is applied. Defaults to
1for a single pass. Values greater than1repeat the cleaning, stopping early once a pass removes no observations. UseInfto repeat until convergence (Repeated Edited Nearest Neighbors).- all_k
A logical. When
TRUE, ENN is applied with an increasing number of neighbors, from1up toneighbors, cleaning the data at each step (All k-Nearest Neighbors). Takes precedence overtimes. Defaults toFALSE.- kind_sel
A character string. The rule used to decide whether an observation is removed.
"mode"(the default) removes an observation when the majority of its neighbors disagree with its class."all"is stricter and removes an observation unless all of its neighbors share its class.- skip
A logical. Should the step be skipped when the recipe is baked by
bake()? While all operations are baked whenprep()is run, some operations may not be able to be conducted on new data (e.g. processing the outcome variable(s)). Care should be taken when usingskip = TRUEas it may affect the computations for subsequent operations.- seed
An integer that will be used as the seed when applied.
- distance_with
A call to a selector function to choose which variables are used for distance calculations. Defaults to
recipes::all_predictors(). The variable selected by...is always excluded from the distance calculations.- id
A character string that is unique to this step to identify it.
Value
An updated version of recipe with the new step
added to the sequence of existing steps (if any). For the
tidy method, a tibble with columns terms which is
the variable used to sample.
Details
Edited Nearest Neighbors (ENN) is a cleaning method. For each observation it
finds the neighbors nearest neighbors and, if the class of the observation
does not match the majority class among those neighbors, the observation is
removed. This tends to remove noisy and borderline observations, which can
lead to smoother decision boundaries.
Setting times greater than 1 applies ENN repeatedly, removing more noisy
and borderline observations on each pass and stopping early once a pass
removes nothing. This corresponds to Repeated Edited Nearest Neighbors
(RENN).
Setting all_k = TRUE applies ENN with increasing numbers of neighbors, from
1 up to neighbors, cleaning the data at each step. This corresponds to
All k-Nearest Neighbors (AllKNN) and takes precedence over times.
Setting kind_sel = "all" uses a stricter cleaning rule: instead of removing
an observation when the majority of its neighbors disagree, it is removed
unless every one of its neighbors shares its class. This removes more
observations than the default kind_sel = "mode".
All variables selected by distance_with must be numeric with no missing
data.
All columns in the data are sampled and returned by recipes::juice()
and recipes::bake().
When used in modeling, users should strongly consider using the
option skip = TRUE so that the extra sampling is not
conducted outside of the training set.
Minimum observations
The data must have at least neighbors + 1 observations for the nearest
neighbors to be computed.
Tidying
When you tidy() this step, a tibble is returned with
columns terms and id:
- terms
character, the selectors or variables selected
- id
character, id of this step
Tuning Parameters
This step has 2 tuning parameters:
neighbors: # Nearest Neighbors (type: integer, default: 3)all_k: Pruning (type: logical, default: FALSE)
Case weights
The underlying operation does not allow for case weights. Supplying data with a case weights column to this step results in an error.
References
Wilson, D. L. (1972). Asymptotic properties of nearest neighbor rules using edited data. IEEE Transactions on Systems, Man, and Cybernetics, (3), 408-421.
Tomek, I. (1976). An experiment with the edited nearest-neighbor rule. IEEE Transactions on Systems, Man, and Cybernetics, (6), 448-452.
See also
enn() for direct implementation
step_smote(), which is commonly composed before step_enn() to clean the
ambiguous points that over-sampling creates near the class boundary (the
equivalent of imbalanced-learn's SMOTEENN).
Other Steps for under-sampling:
step_cluster_centroids(),
step_cnn(),
step_downsample(),
step_instance_hardness(),
step_ncl(),
step_nearmiss(),
step_oss(),
step_tomek()
Examples
library(recipes)
library(modeldata)
data(hpc_data)
hpc_data0 <- hpc_data |>
select(-protocol, -day)
orig <- count(hpc_data0, class, name = "orig")
orig
#> # A tibble: 4 × 2
#> class orig
#> <fct> <int>
#> 1 VF 2211
#> 2 F 1347
#> 3 M 514
#> 4 L 259
up_rec <- recipe(class ~ ., data = hpc_data0) |>
step_enn(class) |>
prep()
training <- up_rec |>
bake(new_data = NULL) |>
count(class, name = "training")
training
#> # A tibble: 4 × 2
#> class training
#> <fct> <int>
#> 1 VF 1737
#> 2 F 731
#> 3 M 262
#> 4 L 173
# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
bake(new_data = hpc_data0) |>
count(class, name = "baked")
baked
#> # A tibble: 4 × 2
#> class baked
#> <fct> <int>
#> 1 VF 2211
#> 2 F 1347
#> 3 M 514
#> 4 L 259
orig |>
left_join(training, by = "class") |>
left_join(baked, by = "class")
#> # A tibble: 4 × 4
#> class orig training baked
#> <fct> <int> <int> <int>
#> 1 VF 2211 1737 2211
#> 2 F 1347 731 1347
#> 3 M 514 262 514
#> 4 L 259 173 259
library(ggplot2)
ggplot(circle_example, aes(x, y, color = class)) +
geom_point() +
labs(title = "Without ENN") +
xlim(c(1, 15)) +
ylim(c(1, 15))
recipe(class ~ x + y, data = circle_example) |>
step_enn(class) |>
prep() |>
bake(new_data = NULL) |>
ggplot(aes(x, y, color = class)) +
geom_point() +
labs(title = "With ENN") +
xlim(c(1, 15)) +
ylim(c(1, 15))