- recipe
A recipe object. The step will be added to the sequence of
operations for this recipe.
- ...
One or more selector functions to choose which
variable is used to sample the data. See recipes::selections
for more details. The selection should result in single
factor variable. For the tidy method, these are not
currently used.
- role
Not used by this step since no new variables are created.
- trained
A logical to indicate if the quantities for preprocessing have
been estimated.
- column
A character string of the variable name that will
be populated (eventually) by the ... selectors.
- over_ratio
A numeric value for the ratio of the
minority-to-majority frequencies. The default value (1) means
that all other levels are sampled up to have the same
frequency as the most occurring level. A value of 0.5 would mean
that the minority levels will have (at most) (approximately)
half as many rows as the majority level.
A named numeric vector can be used instead to give different levels
different targets, for example c(a = 1, b = 0.5). The names must be
levels of the outcome and the values are ratios of the majority level,
exactly as in the single-number case. Levels that are not named are left
untouched, as are rows with a missing outcome. Because a vector of targets
is not a single value, supplying one means this argument can no longer be
tuned. See vignette("ratio", package = "themis") for more details.
- neighbors
An integer. Number of nearest neighbor that are used
to generate the new examples of the minority class.
- indicator_column
A single string or NULL (the default). If a
string is given, a logical column with that name is added to the output,
marking rows added by the step (TRUE) vs rows from the original data
(FALSE).
- skip
A logical. Should the step be skipped when the recipe is baked by
bake()? While all operations are baked when prep() is run, some
operations may not be able to be conducted on new data (e.g. processing the
outcome variable(s)). Care should be taken when using skip = TRUE as it
may affect the computations for subsequent operations.
- seed
An integer that will be used as the seed when applied.
- id
A character string that is unique to this step to identify it.