Learn R Programming

themis (version 1.1.0)

themis-package: themis: Extra Recipes Steps for Dealing with Unbalanced Data

Description

logo

A dataset with an uneven number of cases in each class is said to be unbalanced. Many models produce a subpar performance on unbalanced datasets. A dataset can be balanced by increasing the number of minority cases using SMOTE 2011 tools:::Rd_expr_doi("10.48550/arXiv.1106.1813"), BorderlineSMOTE 2005 tools:::Rd_expr_doi("10.1007/11538059_91") and ADASYN 2008. Or by decreasing the number of majority cases using NearMiss 2003 https://www.site.uottawa.ca/~nat/Workshop2003/jzhang.pdf or Tomek link removal 1976.

Arguments

Author

Maintainer: Emil Hvitfeldt emil.hvitfeldt@posit.co (ORCID)

Authors:

Other contributors:

  • Posit Software, PBC (ROR) [copyright holder, funder]

See Also