These functions define what parameters can be tuned for specific steps.
They also define the recommended objects from the dials package that can
be used to generate new parameter values and other characteristics.
# S3 method for step_adasyn
tunable(x, ...)# S3 method for step_bsmote
tunable(x, ...)
# S3 method for step_cluster_centroids
tunable(x, ...)
# S3 method for step_downsample
tunable(x, ...)
# S3 method for step_enn
tunable(x, ...)
# S3 method for step_instance_hardness
tunable(x, ...)
# S3 method for step_kmeans_smote
tunable(x, ...)
# S3 method for step_ncl
tunable(x, ...)
# S3 method for step_nearmiss
tunable(x, ...)
# S3 method for step_rose
tunable(x, ...)
# S3 method for step_smogn
tunable(x, ...)
# S3 method for step_smote
tunable(x, ...)
# S3 method for step_smoten
tunable(x, ...)
# S3 method for step_smotenc
tunable(x, ...)
# S3 method for step_svmsmote
tunable(x, ...)
# S3 method for step_upsample
tunable(x, ...)
A tibble object.
A recipe step object
Not used.